

#include <ap_int.h>
#include <ap_utils.h>
#include <hls_stream.h>

enum {
	I3_ZERO=0,
	I3_ONE,
	I3_07071,
	I3_05,
};

typedef struct {
	ap_int<3> real;
	ap_int<3> imag;
} cint3;

typedef struct {
//	cint3	   data;
	ap_uint<6> data;
	ap_uint<1> dest;
	ap_uint<1> last;
} cplxi3stream;

typedef struct {
	ap_int<16> real;
	ap_int<16> imag;
} cint16;

typedef struct {
//	cint16	   data;
	ap_uint<32> data;
	ap_uint<1>  last;
} cplxi16stream;

const ap_int<16> f_tab[4] = { 0, 32767, 23170, 0 };

const ap_int<16> remap_int(ap_int<3> x)
{
	if(x<0)
		return -(f_tab[-(x)]);
	return f_tab[x];
}

const ap_uint<32> remap_int2(ap_uint<6> x)
{
	ap_int<3> re=x.range(2,0);
	ap_int<3> im=x.range(5,3);
	cint16 o;
	o.real = remap_int(re);
	o.imag = remap_int(im);
	ap_uint<32> *dO = (ap_uint<32> *) &o;

	return (*dO);
}


/*
 *
  myPosDst: 1
  myPosSrc: 0
  myPosSize: 768
  myNegDst: 1280
  myNegSrc: 768
  myNegSize: 768
  myZeroDst: 769
  myZeroSize: 511

  FFT_REAL(myFftBuffer[0]) = 0;
  FFT_IMAG(myFftBuffer[0]) = 0;
        bzero(&myFftBuffer[769],             511 * sizeof(FFT_TYPE));
        memcpy(&myFftBuffer[1],    &in[0],   768 * sizeof(FFT_TYPE));
        memcpy(&myFftBuffer[1280], &in[768], 768 * sizeof(FFT_TYPE));
 *
 * */

static bool set504 = false;
void pre_ifft_reord(hls::stream<cplxi3stream> &input, hls::stream<cplxi16stream> &output, hls::stream<ap_uint<24> > &out_conf)
{
//#pragma HLS INTERFACE axis port=input
//#pragma HLS INTERFACE axis port=output
//#pragma HLS INTERFACE axis port=out_conf
//#pragma HLS RESOURCE variable=input core=AXI4Stream
//#pragma HLS RESOURCE variable=output core=AXI4Stream
#pragma HLS INTERFACE ap_fifo port=input
#pragma HLS INTERFACE ap_fifo port=output
#pragma HLS INTERFACE ap_fifo port=out_conf
#pragma HLS RESOURCE variable=input  core=AXIS metadata="-bus_bundle S_AXIS"
#pragma HLS RESOURCE variable=output core=AXIS metadata="-bus_bundle M_AXIS"
#pragma HLS RESOURCE variable=out_conf core=AXIS metadata="-bus_bundle M_CONFIG"

#pragma HLS INTERFACE ap_ctrl_none port=return
#pragma HLS PIPELINE off

	cplxi3stream x;
	cplxi16stream y;
	ap_uint<12> i;
	ap_uint<24> confNull;

	x = input.read(); //data=0 -> -0.7071, data=1 -> 0.7071
    if(x.dest) {
        set504 = false;
    	/* configure fft and reorder module */
		confNull = 608; //CP=608
    	out_conf.write(confNull);ap_wait();
		y.data = 0; //will be 0 or 1
	    /* send phase references */
    	for(i=0;i<2048;i++){
    		y.last = (i==2047) ? 1 : 0;
    	    output.write(y);
    	}
    } else {

        if(!set504) {
            /* take back guard interval/CP */
            confNull = 504; //CP=504
            out_conf.write(confNull); ap_wait();
            set504 = true;
        }

#if 1
    	/* positive frequencies */
    	for(i = 0; i < 768; i++) {
    		if(i==0){
                    /* zero frequency */
                    y.data = 0;
                    y.last = 0;
                    output.write(y);
    		} else {
    		    x = input.read();
    		}
    		y.data = remap_int2(x.data);
    		output.write(y);
    	}

    	/* not used frequencies */
    	for(i = 0; i < 511; i++) {
    		y.data = 0; //will be 0 or 1
    		output.write(y);
    	}

    	/* negative frequencies */
    	for(i = 0; i < 768; i++) {
    		x = input.read();
    		y.data = remap_int2(x.data);
    		if(i==767) y.last = 1;
    		output.write(y);
    	}
#else
    	for(i = 0; i < 2048; i++) {
    		if(i==0) {
    			/* zero frequency */
    			y.data.imag = 0; //will be 0 or 1
    			y.data.real = 0; //will be 0 or 1
    			y.last = 0;
    			output.write(y); i++;
        		y.data.imag = remap_int(x.data.imag);
        		y.data.real = remap_int(x.data.real);
    		} else if (i < 769) {
    			x = input.read();
        		y.data.imag = remap_int(x.data.imag);
        		y.data.real = remap_int(x.data.real);
    		} else if(i < 1280){
    			y.data.imag = 0; //will be 0 or 1
    			y.data.real = 0; //will be 0 or 1
    			int x;
    			for(x = 0; x < 511; x++) {
    				output.write(y); i++;
    			}
    		} else {
    			x = input.read();
        		y.data.imag = remap_int(x.data.imag);
        		y.data.real = remap_int(x.data.real);
        		y.last = (i==2047) ? 1 : 0;
    		}
			output.write(y);
    	}
#endif
    }
}
